"use strict";
/**
 * @license
 * Copyright Google LLC All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isFatalDiagnosticError = exports.makeRelatedInformation = exports.makeDiagnostic = exports.FatalDiagnosticError = void 0;
const ts = __importStar(require("typescript"));
const error_code_1 = require("./error_code");
class FatalDiagnosticError {
    code;
    node;
    message;
    relatedInformation;
    constructor(code, node, message, relatedInformation) {
        this.code = code;
        this.node = node;
        this.message = message;
        this.relatedInformation = relatedInformation;
    }
    /**
     * @internal
     */
    _isFatalDiagnosticError = true;
    toDiagnostic() {
        return makeDiagnostic(this.code, this.node, this.message, this.relatedInformation);
    }
}
exports.FatalDiagnosticError = FatalDiagnosticError;
function makeDiagnostic(code, node, messageText, relatedInformation) {
    node = ts.getOriginalNode(node);
    return {
        category: ts.DiagnosticCategory.Error,
        code: (0, error_code_1.ngErrorCode)(code),
        file: ts.getOriginalNode(node).getSourceFile(),
        start: node.getStart(undefined, false),
        length: node.getWidth(),
        messageText,
        relatedInformation,
    };
}
exports.makeDiagnostic = makeDiagnostic;
function makeRelatedInformation(node, messageText) {
    node = ts.getOriginalNode(node);
    return {
        category: ts.DiagnosticCategory.Message,
        code: 0,
        file: node.getSourceFile(),
        start: node.getStart(),
        length: node.getWidth(),
        messageText,
    };
}
exports.makeRelatedInformation = makeRelatedInformation;
function isFatalDiagnosticError(err) {
    return err._isFatalDiagnosticError === true;
}
exports.isFatalDiagnosticError = isFatalDiagnosticError;
