"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImportManager = void 0;
/**
 * @license
 * Copyright Google LLC All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
const typescript_1 = __importDefault(require("typescript"));
const imports_1 = require("../../imports");
class ImportManager {
    rewriter;
    prefix;
    factory;
    specifierToIdentifier = new Map();
    nextIndex = 0;
    constructor(rewriter = new imports_1.NoopImportRewriter(), prefix = 'i', factory = typescript_1.default.factory) {
        this.rewriter = rewriter;
        this.prefix = prefix;
        this.factory = factory;
    }
    generateNamespaceImport(moduleName) {
        // The case `specifierToIdentifier.get(moduleName) === null` is also considered to overwrite the
        // side effect import since namedspace import is enough.
        if (!this.specifierToIdentifier.has(moduleName) ||
            this.specifierToIdentifier.get(moduleName) === null) {
            this.specifierToIdentifier.set(moduleName, this.factory.createIdentifier(`${this.prefix}${this.nextIndex++}`));
        }
        return this.specifierToIdentifier.get(moduleName);
    }
    generateNamedImport(moduleName, originalSymbol) {
        // First, rewrite the symbol name.
        const symbol = this.rewriter.rewriteSymbol(originalSymbol, moduleName);
        // Ask the rewriter if this symbol should be imported at all. If not, it can be referenced
        // directly (moduleImport: null).
        if (!this.rewriter.shouldImportSymbol(symbol, moduleName)) {
            // The symbol should be referenced directly.
            return { moduleImport: null, symbol };
        }
        // If not, this symbol will be imported using a generated namespace import.
        const moduleImport = this.generateNamespaceImport(moduleName);
        return { moduleImport, symbol };
    }
    generateSideEffectImport(moduleName) {
        if (!this.specifierToIdentifier.has(moduleName)) {
            this.specifierToIdentifier.set(moduleName, null);
        }
    }
    getAllImports(contextPath) {
        const imports = [];
        for (const [originalSpecifier, qualifier] of this.specifierToIdentifier) {
            const specifier = this.rewriter.rewriteSpecifier(originalSpecifier, contextPath);
            imports.push({
                specifier,
                qualifier,
            });
        }
        return imports;
    }
}
exports.ImportManager = ImportManager;
